<?php

class Content
{
    static function FindHTMFragmentFor( $page_index, $key_array, $content_path = CONTENT_PATH )
    {
        $page = str_replace( '-index', '', $page_index );

        $keys = "[" . implode( ", ", $key_array ) . "]";

        //error_log( "FindHTMFragmentFor( '$page', $keys, '$content_path' ) [" );

        if ( "index" == $page )
        {
            $htm = self::FindHTMFor( "_index", $key_array, $content_path );
        }
        else
        {
            $htm = "<!-- Could not find:";
            $i   = 0;

            do
            {
               // error_log( "page_index: " . $page_index );

                $htm = self::FindHTMFor( $page, $key_array, $content_path );

                //  page-subpage-subsubpage-index -> page-subpage-index
                if ( ! $htm )
                {
                    $page = self::TruncatePath( $page );
                }

                if ( 100 < $i++ ) break; // Safety latch
            }
            while ( ! $htm && ("" != $page) && ("index" != $page) );
        }

        if ( ! $htm )
        {
            $htm = self::FindHTMFor( "_site", $key_array, $content_path );
        }

        if ( $htm )
        {
            //error_log( "Found: $content_path/$page" . $keys );
        }
        else
        {
            //error_log( "Could not find: $content_path/$page" . $keys );
        }

        return $htm;
    }

    static function TruncatePath( $page_index )
    {
        $bits      = explode( '-', $page_index );
        $last      = count( $bits ) - 1;

        if ( "index" == $bits[$last] )
        {
            $new = array_slice( $bits, 0, -2 );

            $new[] = "index";
        }
        else
        {
            $new = array_slice( $bits, 0, -1 );
        }

        $truncated = join( '-', $new );

        //error_log( "TruncatePath( '$page_index' ) --> '$truncated'" );

        return $truncated;
    }

	static function FindHTMFor( $page_index, $key_array, $content_path = CONTENT_PATH )
	{
		if ( ("dashboard-onboarding-worker" == $page_index) && ("main" == $key_array[0]) )
		{
	        error_log( "FindHTMFor( '$page_index', '" . join( ',', $key_array ) ."', '$content_path' )" );
	    }

		$htm = false;

        //
        //  Keys are imploaded with tab as separator so that trim can be used
        //  to remove any stray outer separators;
        //  then tabs are converted to '-' characters.
        //

		$key = str_replace( "\t", '-', trim( implode( "\t", $key_array ), "\t" ) );

        if ( "" == $key )
        {
            $key_string = implode( "\t", $key_array );
        
            //error_log( "Key string: $key_string" );
            
            $key_trim = trim( $key_string );
            
            //error_log( "Key trim: $key_trim" );
            
            $key_final = str_replace( "\t", '-', $key_trim );
            
            //error_log( "Key final: $key_final" );
        }
        
		if ( self::HasHTMFor( $page_index, $key, $content_path ) )
		{
			$htm = self::getHTMFor( $page_index, $key, $content_path );
		}
		else
		{
            												//						[menu1, CLIENT, principalcorp] 
            $file_prefix = array_slice( $key_array, 0, 1 ); // Remove first element [menu1]
            $options     = array_slice( $key_array, 1 );	// Remove rest					[CLIENT, principalcorp]
            $n           = count( $options );				//								n = 2

            for ( $i = $n - 1; $i >= 0; $i-- )				// for ( $i = 2-1; $i >= 1; $i-- )
            {
                $remaining = array_slice( $options, 0, $i );
                $next_try  = array_merge( $file_prefix, $remaining );

                $htm = self::FindHTMFor( $page_index, $next_try, $content_path );

                if ( $htm ) break;
            }
        }

		return $htm;
	}


	//	<CONTENT PATH>/<page index>/<key>@<server>.htm
	//
	//	<CONTENT PATH>/dashboard/main-CLIENT@healthcare.local.htm
	//
	//	<CONTENT PATH>/dashboard/main-CLIENT@healthcare.local.htm
	//
	//
	//	<CONTENT PATH>/dashboard/main.htm
	//
	//
	//
	//
	static function HasHTMFor( $page_index, $key, $content_path = CONTENT_PATH )
	{
		$server = self::DetermineHostname();

        $found = false;
		$f1    = $content_path . "/" . $page_index . "/" . $key . "@" . $server . ".htm";
		$f2    = $content_path . "/" . $page_index . "/" . $key . ".htm";
		$f     = "";

		if ( file_exists( $f1 ) )
		{
			$found = true;
			$f     = $f1;
		}
		else
		if ( file_exists( $f2 ) )
		{
			$found = true;
			$f     = $f2;
		}

        $result = $found ? "TRUE" : "FALSE";

        if ( $found )
        {
	        error_log( "Using: " . $f );
	        //error_log( "HasHTMFor( '$page_index', '$key', ... ) [$server] : $f : $result" );
	    }
        return $found;
	}

	static function IsHTMFor( $page_index, $key, $content_path = CONTENT_PATH )
	{
		$f1 = $content_path . "/" . $page_index . "/" . $key . ".htm";
		//$f2 = $content_path . "/" . $page_index . "/" . $key . "-" . $_SERVER["SERVER_NAME"] . ".htm";
		
		return file_exists( $f );
	}

	static function getHTMFor( $page_index, $key, $content_path = CONTENT_PATH )
	{
		$h = false;

		$server = self::DetermineHostname();

		$f1    = $content_path . "/" . $page_index . "/" . $key . "@" . $server . ".htm";
		$f2    = $content_path . "/" . $page_index . "/" . $key . ".htm";
		$f     = "";

		if ( file_exists( $f1 ) )
		{
			$h = file_get_contents( $f1, false );
			$f = $f1;
		}
		else		
		if ( file_exists( $f2 ) )
		{
			$h = file_get_contents( $f2, false );
			$f = $f2;
		}

		return $h ? Input::unidecode( $h ) : false;
	}

		static function DetermineHostname()
		{
			return join( ".", array_reverse( explode( ".", $_SERVER["SERVER_NAME"] ) ) );
		}

	static function retrieveArticlesX()
	{
		$articles = array();
		$articles[] = array( "subject" => "English",          "date" => "2013-09-21", "title" => "English Advanced Module A", "views" =>  814 );
		$articles[] = array( "subject" => "Biology",          "date" => "2013-04-27", "title" => "Biology",                   "views" => 3539 );
		$articles[] = array( "subject" => "Business Studies", "date" => "2013-04-21", "title" => "Business Studies",          "views" => 2120 );
		
		return $articles;
	}

	static function retrieveArticles( $search_dir = ARTICLES_PATH )
	{
		$files = Files::recurseFiles( $search_dir, ".htm" );
		$infos = array();

		foreach ( $files as $filepath )
		{
			//error_log( $filepath );

			$infos[] = Article::LoadFrom( $filepath );
		}

		return $infos;
	}

	static function retrieveArticlesByCategory( $category, $search_dir = ARTICLES_PATH )
	{
		$search_dir = $search_dir . "/$category";

		$files = Files::recurseFiles( $search_dir, ".htm" );
		$infos = array();

		foreach ( $files as $filepath )
		{
			$infos[] = Article::LoadFrom( $filepath );
		}

		return $infos;
	}

	static function retrieveModArticles( $page, $mod )
	{
		$nr = $page->getRequest( "display" );
		$nr = $nr ? $nr : $mod;
	
		$search_dir = ARTICLES_PATH;

		$files = Files::recurseFiles( $search_dir, ".htm" );
		$infos = array();

		foreach ( $files as $filepath )
		{
			error_log( $filepath );
		
			$infos[] = ArticleInfo::Decode( $filepath );
		}

		$sorted = Content::SortByDate( $infos );

		return array_slice( $sorted, 0, $nr );
	}

	static function retrieveNArticles( $page, $nr, $pg )
	{
		$i = $nr * ($pg - 1);

		$search_dir = ARTICLES_PATH;

		$files = Files::recurseFiles( $search_dir, ".htm" );
		$infos = array();

		foreach ( $files as $filepath )
		{
			error_log( $filepath );
		
			$infos[] = ArticleInfo::Decode( $filepath );
		}

		$sorted = Content::SortByDate( $infos );

		return array_slice( $sorted, $i, $nr );
	}

	static function encodeQuotes( $h )
	{
		$h = str_replace( " '", " &lsquo;", $h );
		$h = str_replace( "' ", "&rsquo; ", $h );
		$h = str_replace(  "'",  "&rsquo;", $h );

		return $h;
	}

	static function SortRandom( $articles )
	{
		$dict   = array();
		$keys   = array();
		$sorted = array();

		foreach ( $articles as $article )
		{
			$key = $article->getPubDate();
		
			if ( ! array_key_exists( $key, $dict ) ) $keys[] = $key;
			
			$dict[$key][] = $article;
		}

		shuffle( $keys );

		foreach ( $keys as $key )
		{
			$array = $dict[$key];
			
			foreach ( $array as $a )
			{
				$sorted[] = $a;
			}
		}
		
		return $sorted;
	}

	static function SortByDate( $articles )
	{
		$dict   = array();
		$keys   = array();
		$sorted = array();

		foreach ( $articles as $article )
		{
			$key = $article->getPubDate();
		
			if ( ! array_key_exists( $key, $dict ) ) $keys[] = $key;
			
			$dict[$key][] = $article;
		}

		arsort( $keys );

		foreach ( $keys as $key )
		{
			$array = $dict[$key];
			
			foreach ( $array as $a )
			{
				$sorted[] = $a;
			}
		}
		
		return $sorted;
	}

	static function SortByReverseDate( $articles )
	{
		$dict   = array();
		$keys   = array();
		$sorted = array();

		foreach ( $articles as $article )
		{
			$key = $article->getPubDate();
		
			if ( ! array_key_exists( $key, $dict ) ) $keys[] = $key;
			
			$dict[$key][] = $article;
		}

		asort( $keys );

		foreach ( $keys as $key )
		{
			$array = $dict[$key];
			
			foreach ( $array as $a )
			{
				$sorted[] = $a;
			}
		}
		
		return $sorted;
	}

	static function SortByAuthor( $articles )
	{
		$articles = Content::SortByDate( $articles );

		$dict   = array();
		$keys   = array();
		$sorted = array();

		foreach ( $articles as $article )
		{
			$key = trim( str_replace( 'Dr', '', str_replace( 'Dr.', '', $article->getAuthor() ) ) );

			if ( ! array_key_exists( "$key", $dict ) ) $keys[] = $key;
		
			$dict[$key][] = $article;
		}

		asort( $keys );

		foreach ( $keys as $key )
		{
			$array = $dict[$key];
			
			foreach ( $array as $a )
			{
				$sorted[] = $a;
			}
		}
		
		return $sorted;
	}

	static function SortByTitle( $articles )
	{
		$articles = Content::SortByDate( $articles );

		$dict   = array();
		$keys   = array();
		$sorted = array();

		foreach ( $articles as $article )
		{
			$key = $article->getTitle();

			if ( ! array_key_exists( "$key", $dict ) ) $keys[] = $key;
		
			$dict[$key][] = $article;
		}

		asort( $keys );

		foreach ( $keys as $key )
		{
			$array = $dict[$key];
			
			foreach ( $array as $a )
			{
				$sorted[] = $a;
			}
		}
		
		return $sorted;
	}

	static function SortByArticleID( $articles )
	{
		$articles = Content::SortByDate( $articles );

		$dict   = array();
		$keys   = array();
		$sorted = array();

		foreach ( $articles as $article )
		{
			$key = $article->getArticleID();

			if ( ! array_key_exists( "$key", $dict ) ) $keys[] = $key;
		
			$dict[$key][] = $article;
		}

		asort( $keys );

		foreach ( $keys as $key )
		{
			$array = $dict[$key];
			
			foreach ( $array as $a )
			{
				$sorted[] = $a;
			}
		}
		
		return $sorted;
	}

	static function SortByViews( $articles, $counts )
	{
		//$articles = Content::SortByDate( $articles );

		$dict   = array();
		$keys   = array();
		$sorted = array();

		foreach ( $articles as $article )
		{
			$id  = $article->getResourceID();

			if ( array_key_exists( $id, $counts ) )
			{
				$tuple = array_get( $counts, $id );

				$key = array_get( $tuple, "nr_of_views" );
			}
			else
			{
				$key = "0";
			}

			if ( ! array_key_exists( "$key", $dict ) ) $keys[] = $key;
		
			$dict[$key][] = $article;
		}

		arsort( $keys );

		foreach ( $keys as $key )
		{
			$array = $dict[$key];
			
			foreach ( $array as $article )
			{
				$sorted[] = $article;

				error_log( "Adding: " . $article->getArticleID() );
			}
		}
		
		return $sorted;
	}
	
	static function filter( $all_articles, $keyword )
	{
		$keyword = strtolower( $keyword );
		$filtered = array();
		
		foreach ( $all_articles as $article )
		{
			if ( string_contains( strtolower( $article->getTitle() ), $keyword ) || string_contains( strtolower( $article->getCategory() ), $keyword ) )
			{
				$filtered[] = $article;
			}
		}
	
		return $filtered;
	}
}

?>
